@@ -16,17 +16,33 @@
 
 import collections
 import copy
+import enum
 import io
 import logging
 import os
+import pathlib
 import re
 import textwrap
-import enum
 import urllib.parse
-import pathlib
-
-from typing import TYPE_CHECKING, Dict, List, Iterator, Tuple, Optional, Set, TextIO, Union, Match, cast, AsyncIterator
+from typing import (
+    TYPE_CHECKING,
+    AsyncIterator,
+    Dict,
+    Iterator,
+    List,
+    Match,
+    Optional,
+    Set,
+    TextIO,
+    Tuple,
+    Union,
+    cast,
+)
 
+import requests
+from pygls.lsp.types import Location, Position, Range
+from pygls.server import LanguageServer
+from pygls.workspace import Document
 from zc.buildout.buildout import _buildout_default_options
 from zc.buildout.configparser import (
     MissingSectionHeaderError,
@@ -35,13 +51,8 @@ from zc.buildout.configparser import (
     option_start,
     section_header,
 )
-import requests
-from . import recipes
-from . import jinja
 
-from pygls.server import LanguageServer
-from pygls.workspace import Document
-from pygls.lsp.types import Position, Range, Location
+from . import jinja, recipes
 
 logger = logging.getLogger(__name__)
 requests_session = requests.Session()
@@ -63,6 +74,9 @@ section_reference_re = re.compile(r'.*\$\{(?P<section>[-a-zA-Z0-9 ._]*)[^:]*$')
 option_definition_re = re.compile(
     r'^(?P<option>[^=]*)\s*=\s*(?P<option_value>.*)$')
 
+# Matches a comment
+comment_re = re.compile(r'.*[#;].*')
+
 # Filenames of slapos instances, that might be a buildout profile as a buildout template
 slapos_instance_profile_filename_re = re.compile(
     r'.*\/instance[^\/]*\.cfg[^\/]*')
@@ -214,6 +228,7 @@ class SymbolKind(enum.Enum):
        the position is on ``section`` from ``${section:option}``.
     * ``OptionReference``: a specialised version of ``BuildoutOptionValue`` where
        the position is on ``option`` from ``${section:option}``.
+    * ``Comment``: when inside a comment
 
   """
   SectionDefinition = 0
@@ -221,6 +236,7 @@ class SymbolKind(enum.Enum):
   BuildoutOptionValue = 2
   SectionReference = 3
   OptionReference = 4
+  Comment = 5
 
 
 class Symbol:
@@ -339,6 +355,9 @@ class BuildoutTemplate:
     if position.line < len(lines):
       line = lines[position.line]
 
+    if comment_re.match(line[:position.character]):
+      return Symbol(kind=SymbolKind.Comment, buildout=self.buildout, value='')
+
     line_offset = 0
     remaining_line = line
     while remaining_line:
@@ -527,31 +546,8 @@ class BuildoutProfile(Dict[str, BuildoutSection], BuildoutTemplate):
         for template_option_name in recipe.template_options:
           template_option_value = section_value.get(template_option_name)
           if template_option_value is not None:
-            template_option_value_uri = template_option_value.value
-            # expand substitutions
-            if '${' in template_option_value_uri:
-              section_value_dict = dict(section_value)
-              if '<' in section_value:
-                section_value_dict = dict(self[section_value['<'].value])
-                section_value_dict.update(dict(section_value))
-              logger.debug("We have a section reference %s in %s",
-                           template_option_value_uri, section_value_dict)
-
-              def expand_section_reference(match: Match[str]) -> str:
-                referenced_section_name = match.group(
-                    'section') or section_name
-                if referenced_section_name in self:
-                  referenced_section = self[referenced_section_name]
-                  if match.group('option') in referenced_section:
-                    return referenced_section[match.group('option')].value
-                return '\0'  # won't likely match a filename
-
-              template_option_value_uri = option_reference_strict_re.sub(
-                  expand_section_reference,
-                  template_option_value_uri,
-              )
-              logger.debug("Section reference expanded to: %s",
-                           template_option_value_uri)
+            template_option_value_uri = self.resolve_value(
+                section_name, template_option_name)
 
             # Normalize URI path, in case it contain double slashes, ./ or ..
             template_option_value_parsed = urllib.parse.urlparse(
@@ -802,6 +798,52 @@ class BuildoutProfile(Dict[str, BuildoutSection], BuildoutTemplate):
     return (uri.endswith('.cfg') or uri.endswith('.cfg.in')
             or uri.endswith('.cfg.j2') or uri.endswith('.cfg.jinja2'))
 
+  def resolve_value(self, section_name: str, option_name: str) -> str:
+    """Get the value of an option, after substituting references.
+
+    If substitution is not possible, the original value is returned.
+    """
+    def _get_section(section_name: str) -> BuildoutSection:
+      section = self[section_name]
+      if '<' in section:
+        macro = copy.copy(self[section['<'].value])
+        macro.update(**section)
+        return macro
+      return section
+
+    def _resolve_value(
+        section_name: str,
+        option_name: str,
+        value: str,
+        seen: Set[Tuple[str, str]],
+    ) -> str:
+      if (section_name, option_name) in seen:
+        return value
+      seen.add((section_name, option_name))
+
+      def _sub(match: Match[str]) -> str:
+        referenced_section_name = match.group('section') or section_name
+        if referenced_section_name in self:
+          referenced_section = _get_section(referenced_section_name)
+          referenced_option = match.group('option')
+          if referenced_option in referenced_section:
+            return _resolve_value(
+                referenced_section_name,
+                referenced_option,
+                referenced_section[referenced_option].value,
+                seen,
+            )
+        return value
+
+      return option_reference_strict_re.sub(_sub, value)
+
+    return _resolve_value(
+        section_name,
+        option_name,
+        _get_section(section_name)[option_name].value,
+        set(),
+    )
+
 
 class ResolvedBuildout(BuildoutProfile):
   """A buildout where extends and section macros <= have been extended.
